/* Main.ino file generated by New Project wizard
 *
 * Created:   Mon Sept 2024
 * Processor: Arduino Uno
 * Compiler:  Arduino AVR
 */

// Code based to "MPPT for PV System Based on Variable Step Size Perturb and Observe Algorithm" paper
// by Awang Jusoh, Rozana Alik, Tan Kar Guan, Tole Sutikno. 
// http://telkomnika.uad.ac.id/index.php/TELKOMNIKA/article/view/3160
 
#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 10, 11, 12, 13);

#define PWMOUT       5
#define PVPVOLTAGE   A0
#define PVPCURRENT   A1
#define LOADVOLTAGE  A2
#define LOADCURRENT  A3

float Voltage = 0.0;
float Current = 0.0;
float loadVoltage = 0.0;
float loadCurrent = 0.0;
float loadPower = 0.0; 
float Power = 0.0;
float prevPower = 0.0, prevCurrent = 0.0, prevVoltage = 0.0, dI = 0.0, dV = 0.0 ; 
float deltaPower = 1.5;                 
float pwmMin = 10.0, pwmMax = 240.0;
float pwmValue = 128.0;                        
float efficiency;

void setup() 
 { pinMode(PWMOUT, OUTPUT);
   lcd.begin(20, 4);
 }

void loop() 
 { // Get PV panel Voltage and Current:
   Voltage = (analogRead(PVPVOLTAGE) * 5.0 /1024) * 5.0;
   Current = (analogRead(PVPCURRENT) * 5.0 /1024);
   // Compute PV panel power.
   Power = Voltage * Current;

   // Get Voltage, Current on the Load:
   loadVoltage = (analogRead(LOADVOLTAGE) * 5.0 /1024) * 11.0;
   loadCurrent = (analogRead(LOADCURRENT) * 5.0 /1024);
   // Compute power on the load.
   loadPower = loadVoltage * loadCurrent;
   
   // Compute efficiency:
   efficiency = loadPower/Power*100;
   if (efficiency>100.0)
      efficiency = 100.0;
   
   // Implementation of the MPPT P&O (Perturb and Observe) Algorithm
   if (Power > prevPower)
    { if (Voltage > prevVoltage)
         pwmValue = pwmValue - deltaPower;
      else
         pwmValue = pwmValue + deltaPower;
    }
   else
    { if (Voltage > prevVoltage)
         pwmValue = pwmValue + deltaPower;
      else
         pwmValue = pwmValue - deltaPower;
    }
   prevPower = Power;
   prevVoltage = Voltage;
  
   if (pwmValue < pwmMin)
      pwmValue = pwmMin;
   if (pwmValue > pwmMax)
      pwmValue = pwmMax;

   // Output PWM new value.
   analogWrite(PWMOUT, pwmValue);
   
   // Display the parameters:
   doDisplay();   
 }

void doDisplay()
 { lcd.setCursor(1, 0);
   lcd.print("Ppv="), lcd.print(Power, 1),    lcd.print("W");   
   lcd.setCursor(11, 0);   
   lcd.print("PWM="), lcd.print(pwmValue/255*100, 0), lcd.print(" ");
   lcd.setCursor(1, 1);
   lcd.print("Vpv="), lcd.print(Voltage, 1),  lcd.print("V");
   lcd.setCursor(11, 1);   
   lcd.print("I="),   lcd.print(Current, 1),  lcd.print("A ");
   // Display Load parameters: load Voltage(V), Current(A), Power(W) and efficiency(%).
   lcd.setCursor(1, 2);
   lcd.print("VL="), lcd.print(loadVoltage, 1), lcd.print("V");
   lcd.setCursor(11, 2);   
   lcd.print("IL="), lcd.print(loadCurrent, 1), lcd.print("A ");
   lcd.setCursor(1, 3);
   lcd.print("PL="), lcd.print(loadPower, 1), lcd.print("W ");
   lcd.setCursor(11, 3);
   lcd.print("eff="), lcd.print(efficiency, 1), lcd.print("% ");
 }